@@ -97,7 +97,6 @@ sub new
 
 =head2 listTypes
 
-
 Returns an array of list files supported (currently 'movies', 'directors', 'actors', 'actresses', 'genres', 'ratings', 'keywords', 'plot')
 
 =cut
@@ -138,28 +137,49 @@ sub withThousands ($)
     return $val;
 }
 
-use XMLTV::Gunzip;
-use IO::File;
+use constant Have_gunzip => eval {
+    require IO::Uncompress::Gunzip;
+};
+
 
 sub openMaybeGunzip($)
 {
-    for ( shift ) {
-	return gunzip_open($_) if m/\.gz$/;
-	return new IO::File("< $_");
+    my ($file)=@_;
+    
+    if ($file=~m/\.gz$/ ) {
+	if ( Have_gunzip ) {
+	    return new IO::Uncompress::Gunzip($file);
+	}
+	else {
+	    my $fd;
+
+	    if ( open($fd, "gzip -d < $file |") ) {
+		return($fd);
+	    }
+	    carp("no suitable gzip decompression found");
+	}
+    }
+    else {
+	require IO::File;
+	return new IO::File("< $file");
     }
 }
 
 sub closeMaybeGunzip($$)
 {
-    if ( $_[0]=~m/\.gz$/o ) {
-	# Would close($fh) but that causes segfaults on my system.
-	# Investigating, but in the meantime just leave it open.
-	#
-	#return gunzip_close($_[1]);
-    }
+    my ($file, $fd)=@_;
 
-    # Apparently this can also segfault (wtf?).
-    #return close($_[1]);
+    if ($file=~m/\.gz$/ ) {
+	if ( Have_gunzip ) {
+	    $fd->close();
+	}
+	else {
+	    close($fd);
+	}
+    }
+    else {
+	$fd->close();
+    }
 }
 
 
@@ -252,9 +272,6 @@ sub decodeImdbKey($$$)
 	    $parentKey=~s/\" \(/ \(/o;
 
 	    if ( defined($self->{seriesKeys}->{$parentKey}) ) {
-		#if ( $parentKey=~m/24 Hour Design/o ) {
-		#warn(" lok: $parentKey");
-		#}
 		$hash{parentId}=$self->{seriesKeys}->{$parentKey};
 	    }
 	}
@@ -275,9 +292,6 @@ sub decodeImdbKey($$$)
 	else {
 	    if ( $hash{parentId} == 0 ) {
 		$hash{qualifier}=$self->{QualifierTypeIDs}->{"tv_series"};
-		#if ( $dbkey=~m/24 Hour Design/o ) {
-		    #warn(" key: $dbkey");
-		#}
 		$self->{seriesKeys}->{$dbkey}=$titleID;
 	    }
 	    else {
@@ -457,23 +471,7 @@ sub importMovies($$$$)
 				      $decoded->{parentId},
 				      $decoded->{series},
 				      $decoded->{episode});
-	    if ( $mkey=~m/24 Hour Design/o ) {
-		warn("",join(',', $count,
-			     $decoded->{searchTitle},
-			     $decoded->{title},
-			     $decoded->{qualifier},
-			     $decoded->{year},
-			     $decoded->{parentId},
-			     $decoded->{series},
-			     $decoded->{episode}));
-	    }
 	    
-	    #$DB->insert_db_raw();
-	    #my $titleID=$DB->insert_db('Titles', 'TitleID', IMDBKey=>$mkey,
-	    #Title=>$decoded->{title},
-	    #	       Episode=>$decoded->{episode},
-	    #		       Year=>$decoded->{year},
-	    #		       QualifierTypeID=>$decoded->{qualifier});
 	    $self->{imdbMovie2DBKey}->{$mkey}=$count;
 
 	    #if ( ($count % 50000) == 0 ) {
@@ -501,7 +499,7 @@ sub importMovies($$$$)
 
     $progress->update($countEstimate) if ($self->{showProgressBar});
 
-    $self->status(sprintf("importing Movies found ".withThousands($count)." titles in ".
+    $self->status(sprintf("importing Movies found ".withThousands($count)." in ".
 			  withThousands($lineCount)." lines in %d seconds",time()-$startTime));
 
     closeMaybeGunzip($file, $fh);
@@ -590,7 +588,7 @@ sub importGenres($$$$)
 	    if ( $self->{imdbMovie2DBKey}->{$mkey} ) {
 		# insert into db as discovered
 		if ( ! defined($self->{GenreID}->{$genre}) ) {
-		    $self->{GenreID}->{$genre}=$DB->insert_db('Genres', 'GenreID', Name=>$genre);
+		    $self->{GenreID}->{$genre}=$DB->insert_row('Genres', 'GenreID', Name=>$genre);
 		}
 		$tableInsert_sth->execute($self->{imdbMovie2DBKey}->{$mkey},
 					  $self->{GenreID}->{$genre});
@@ -623,7 +621,7 @@ sub importGenres($$$$)
 
     $progress->update($countEstimate) if ($self->{showProgressBar});
 
-    $self->status(sprintf("importing Genres found ".withThousands($count)." titles in ".
+    $self->status(sprintf("importing Genres found ".withThousands($count)." in ".
 			  withThousands($lineCount)." lines in %d seconds",time()-$startTime));
 
     closeMaybeGunzip($file, $fh);
@@ -632,10 +630,10 @@ sub importGenres($$$$)
 
 sub importActors($$$$)
 {
-    my ($self, $whichCastOrDirector, $castCountEstimate, $file, $DB)=@_;
+    my ($self, $whichCastType, $castCountEstimate, $file, $DB)=@_;
     my $startTime=time();
 
-    if ( $whichCastOrDirector eq "Actors" ) {
+    if ( $whichCastType eq "Actors" ) {
 	if ( $DB->table_row_count('Actors') > 0 ||
 	     $DB->table_row_count('Titles2Actors') > 0 ) {
 	    $self->status("clearing previously loaded data..");
@@ -648,11 +646,11 @@ sub importActors($$$$)
     my $whatAreWeParsing;
     my $lineCount=0;
 
-    if ( $whichCastOrDirector eq "Actors" ) {
+    if ( $whichCastType eq "Actors" ) {
 	$header="THE ACTORS LIST";
 	$whatAreWeParsing=1;
     }
-    elsif ( $whichCastOrDirector eq "Actresses" ) {
+    elsif ( $whichCastType eq "Actresses" ) {
 	$header="THE ACTRESSES LIST";
 	$whatAreWeParsing=2;
     }
@@ -661,7 +659,7 @@ sub importActors($$$$)
     }
 
     my $fh = openMaybeGunzip($file) || return(-2);
-    my $progress=Term::ProgressBar->new({name  => "importing $whichCastOrDirector",
+    my $progress=Term::ProgressBar->new({name  => "importing $whichCastType",
 					 count => $castCountEstimate,
 					 ETA   => 'linear'})
       if ($self->{showProgressBar});
@@ -734,7 +732,10 @@ sub importActors($$$$)
 	    $castNames++;
 
 	    $cur_actorId++;
-	    $tableInsert_sth->execute($cur_actorId, MakeSearchtitle($DB, $cur_name, 0), $cur_name);
+
+	    my $c=$cur_name;
+	    $c=~s/\s*\([IVX]+\)//o;
+	    $tableInsert_sth->execute($cur_actorId, MakeSearchtitle($DB, $c, 0), $cur_name);
 
 	    if ( $self->{showProgressBar} ) {
 		# re-adjust target so progress bar doesn't seem too wonky
@@ -803,7 +804,7 @@ sub importActors($$$$)
     }
     $progress->update($castCountEstimate) if ($self->{showProgressBar});
 
-    $self->status(sprintf("importing $whichCastOrDirector found ".withThousands($castNames)." names, ".
+    $self->status(sprintf("importing $whichCastType found ".withThousands($castNames)." names, ".
 			  withThousands($count)." titles in ".withThousands($lineCount)." lines in %d seconds",time()-$startTime));
     
     closeMaybeGunzip($file, $fh);
@@ -919,7 +920,10 @@ sub importDirectors($$$)
 
 	    if ( !defined($found{$cur_name}) ) {
 		$found{$cur_name}=$directorCount++;
-		$tableInsert_sth->execute($directorCount, MakeSearchtitle($DB, $cur_name, 0), $cur_name);
+
+		my $c=$cur_name;
+		$c=~s/\s*\([IVX]+\)//o;
+		$tableInsert_sth->execute($directorCount, MakeSearchtitle($DB, $c, 0), $cur_name);
 	    }
 
 	    $tableInsert_sth2->execute($self->{imdbMovie2DBKey}->{$line}, $found{$cur_name});
@@ -1030,7 +1034,7 @@ sub importRatings($$)
     }
     $progress->update($countEstimate) if ($self->{showProgressBar});
 
-    $self->status(sprintf("importing Ratings found ".withThousands($count)." titles in ".
+    $self->status(sprintf("importing Ratings found ".withThousands($count)." in ".
 			  withThousands($lineCount)." lines in %d seconds",time()-$startTime));
     
     closeMaybeGunzip($file, $fh);
@@ -1104,11 +1108,11 @@ sub importKeywords($$$$)
 		    if ( !defined($found{$keyword}) ) {
 			$found{$keyword}=$keywordCount++;
 			$tableInsert_sth->execute($keywordCount, $keyword);
-			#=$DB->insert_db('Keywords', 'KeywordID', Name=>$keyword);
+			#=$DB->insert_row('Keywords', 'KeywordID', Name=>$keyword);
 		    }
 		    $tableInsert_sth2->execute($self->{imdbMovie2DBKey}->{$title}, $found{$keyword});
 
-		    #$DB->insert_db('Titles2Keywords', undef, TitleID=>$self->{imdbMovie2DBKey}->{$title}, KeywordID=>$found{$keyword});
+		    #$DB->insert_row('Titles2Keywords', undef, TitleID=>$self->{imdbMovie2DBKey}->{$title}, KeywordID=>$found{$keyword});
 		    $countImported++;
 		    if ( ($countImported % 50000) == 0 ) {
 			$DB->commit();
@@ -1134,7 +1138,7 @@ sub importKeywords($$$$)
     }
     $progress->update($countEstimate) if ($self->{showProgressBar});
 
-    $self->status(sprintf("importing Keywords found ".withThousands($count)." titles in ".
+    $self->status(sprintf("importing Keywords found ".withThousands($count)." in ".
 			  withThousands($lineCount)." lines in %d seconds",time()-$startTime));
 
     closeMaybeGunzip($file, $fh);
@@ -1246,7 +1250,7 @@ sub importPlots($$$$)
     }
     $progress->update($countEstimate) if ($self->{showProgressBar});
 
-    $self->status(sprintf("importing Plots found $count ".withThousands($count)." in ".
+    $self->status(sprintf("importing Plots found ".withThousands($count)." in ".
 			  withThousands($lineCount)." lines in %d seconds",time()-$startTime));
 
     closeMaybeGunzip($file, $fh);
@@ -1728,7 +1732,7 @@ sub _NOT_USED_checkSantity($)
 
 =head1 AUTHOR
 
-jerryv, C<< <jerryv at cpan.org> >>
+jerryv, C<< <jerry.veldhuis at gmail.com> >>
 
 =head1 BUGS
 
