@@ -3,10 +3,11 @@ package IMDB::Local::DB::Base;
 use 5.006;
 use strict;
 use warnings;
+use Carp;
 
 =head1 NAME
 
-IMDB::Local::DB::Base - The great new IMDB::Local::DB::Base!
+IMDB::Local::DB::Base - database abstraction with handy entry points
 
 =head1 VERSION
 
@@ -19,19 +20,8 @@ our $VERSION = '0.01';
 
 =head1 SYNOPSIS
 
-Quick summary of what the module does.
+This module presents one set of handy subroutines around a DBI connection. The interface is not well document, only as much as it is needed for the IMDB::Local::DB developer requires. It is meant to be extended.
 
-Perhaps a little code snippet.
-
-    use IMDB::Local::DB::Base;
-
-    my $foo = IMDB::Local::DB::Base->new();
-    ...
-
-=head1 EXPORT
-
-A list of functions that can be exported.  You can delete this section
-if you don't export anything, such as for a purely object-oriented module.
 
 =head1 SUBROUTINES/METHODS
 
@@ -40,14 +30,14 @@ if you don't export anything, such as for a purely object-oriented module.
 =cut
 
 use DBI;
-use Carp;
 use Time::HiRes;
+use IMDB::Local::DB::RecordIterator;
 
 use Class::MethodMaker
     [ 
       scalar => ['dbh', 'modCount'],
       scalar => ['driver'],
-      scalar => ['driverDetail'],
+      scalar => [{-default => ''}, 'driverDetail'],
       scalar => ['server'],
       scalar => ['database'],
       scalar => ['user'],
@@ -226,11 +216,12 @@ sub connect($)
 	$self->mutex(undef);
 	return(undef);
     }
+
     #print STDERR "autocommit: ".$dbh->{AutoCommit}."\n";
 
     # default for ODC is 80 bytes for LongReadLen, which causes
     # ODBC SQL Server Driver String data, right truncation (SQL-01004)
-    # failures for things like xml data types such as ScheduledJobs.Config
+    # failures for things with lasrge enough strings
     #
     # note: setting arbitrarily high # causes memory allocation problems.
     $dbh->{LongReadLen}   = $self->maxReadLen();
@@ -259,6 +250,9 @@ sub connect($)
 
 =head2 disconnect
 
+Disconnect from the database. Note that for those lazy programmers that fail to call disconnect, the disconnect will be called when the
+object is destructed through perl's DESTROY.
+
 =cut
 
 sub disconnect()
@@ -285,18 +279,25 @@ sub disconnect()
 
 =head2 commit
 
+Commit a DBI transaction (should only be used if db_AutoCommit was zero).
+
 =cut
 
 sub commit()
 {
     my ($self)=@_;
 
+    if ( ! defined($self->dbh) ) {
+	carp("not connected");
+    }
     $self->dbh->commit();
     $self->modCount(0);
 }
 
 =head2 isConnected
 
+Check to see if there has been previous successful 'connect' call.
+
 =cut
 
 sub isConnected($)
@@ -311,17 +312,27 @@ sub isConnected($)
 sub quote($)
 {
     my ($self, @rest)=@_;
+
+    if ( ! defined($self->dbh) ) {
+	carp("not connected");
+    }
     return($self->dbh->quote(@rest));
 }
 
 =head2 last_inserted_key
 
+Retrieve the last inserted key for a given table and primaryKey.
+
 =cut
 
 sub last_inserted_key($$$)
 {
     my ($self, $table, $primaryKey)=@_;
-    #return $self->dbh->last_insert_id(undef, undef, $table, $primaryKey);
+
+    if ( ! defined($self->dbh) ) {
+	carp("not connected");
+    }
+
     if ( $self->driver eq 'ODBC' ) {
 	return $self->select2Scalar("select \@\@identity");
     }
@@ -330,13 +341,20 @@ sub last_inserted_key($$$)
 
 =head2 runSQL
 
+Execute a sql statement and return 1 upon success and 0 upon success. Upon failure, carp() is called with the sql statement.
+
 =cut
 
 sub runSQL($)
 {
     my ($self, $stmt)=@_;
 
+    if ( ! defined($self->dbh) ) {
+	carp("not connected");
+    }
+
     my $dbh=$self->dbh();
+
     #warn("$stmt");
     $dbh->do($stmt);
     if ( $dbh->err() ) {
@@ -348,33 +366,50 @@ sub runSQL($)
 
 =head2 runSQL_err
 
+Return DBI->err() to retrieve error status of previous call to runSQL.
+
 =cut
 
 sub runSQL_err($)
 {
     my ($self)=@_;
+
+    if ( ! defined($self->dbh) ) {
+	carp("not connected");
+    }
+
     return $self->dbh()->err();
 }
 
 =head2 runSQL_srrstr
 
+Return DBI->errstr() to retrieve error status of previous call to runSQL.
+
 =cut
 
 sub runSQL_errstr($)
 {
     my ($self)=@_;
+
+    if ( ! defined($self->dbh) ) {
+	carp("not connected");
+    }
+
     return $self->dbh()->errstr();
 }
 
 =head2 prepare
 
+Return DBI->prepare() for a given statement.
+
 =cut
 
 sub prepare($$)
 {
     my ($self, $query)=@_;
+
     if ( !defined($self->dbh) ) {
-	warn("attempt to prepare statement without db connection");
+	carp("not connected");
     }
     #warn("STMT:$query");
     return $self->dbh->prepare($query);
@@ -382,6 +417,9 @@ sub prepare($$)
 
 =head2 execute
 
+Wrapper for calling DBI->prepare() and DBI->exeute() fora given query. Upon success the DBI->prepare() handle is returned.
+Upon failure, warn() is called with the query statement string and undef is returned.
+
 =cut
 
 sub execute($)
@@ -389,8 +427,7 @@ sub execute($)
     my ($self, $query)=@_;
 
     if ( !defined($self->dbh) ) {
-	warn("attempt to prepare statement without db connection");
-	return(undef);
+	carp("not connected");
     }
 
     #warn("STMT:$query");
@@ -406,31 +443,25 @@ sub execute($)
     return($sth);
 }
 
-=head2 sth_columns
+=head2 insert_row
+
+Execute a table insertion and return the created primaryKey (if specified).
+If primaryKey is not defined, 1 is returned upon success.
 
 =cut
 
-sub sth_columns($$)
-{
-    my ($self, $sth)=@_;
-    my @columns;
-
-    for (my $i=0; $i<$sth->{NUM_OF_FIELDS}; $i++) {
-	push(@columns, $sth->{NAME}->[$i]);
-    }
-    return(@columns);
-}
-
-=head2 insert_db
-
-=cut
-
-sub insert_db($$$%)
+sub insert_row($$$%)
 {
     my ($self, $table, $primaryKey, %args) = @_;
 
+    my $dbh=$self->dbh();
+    if ( !defined($dbh) ) {
+	carp("not connected");
+    }
+
     my ($k,$v)=('','');
     my @values;
+
     for my $key (sort keys %args) {
 	if ( defined($args{$key}) ) {
 	    $k.=$key.",";
@@ -445,11 +476,11 @@ sub insert_db($$$%)
 	warn "attempt to insert nothing into table $table";
 	return(undef);
     }
+
     my $stmt="INSERT INTO $table ($k) VALUES ($v)";
     #print STDERR "STMT:$stmt\n";
     #print STDERR "VALUES:(".join(',', map($self->dbh->quote($_), @values)).")\n";
     
-    my $dbh=$self->dbh();
     my $sth=$dbh->prepare($stmt);
     $sth->execute(@values);
 
@@ -483,12 +514,24 @@ sub _quoteField($$)
 	    return($1.'['.$2.']');
 	}
     }
+
     # no quote needed
     return($word);
 }
 
 =head2 query2SQLStatement
 
+Construct an sql query using a hash containing:
+
+  fields - required array of fields to select
+  tables - required array of tables to select from
+  wheres - optional array of where clauses to include (all and'd together)
+  groupbys - optional array of group by clauses to include
+  sortByField - optional field to sort by (if prefixed with -, then sort is reversed)
+  orderbys - optional array of order by clauses to include
+  offset - offset of returned rows
+  limit - optional integer value to limit # of returned rows
+
 =cut
 
 sub query2SQLStatement($%)
@@ -579,6 +622,11 @@ sub query2SQLStatement($%)
 
 =head2 findRecords
 
+Call query2SQLStatement with the given hash arguments and return a IMDB::Local::DB::RecordIterator handle.
+
+In addition to the query2SQLStatement arguments the following are optional:
+  cacheBy - set cacheBy value in returned IMDB::Local::DB::RecordIterator handle. If not specified, limit is used.
+
 =cut
 
 sub findRecords($%)
@@ -608,19 +656,30 @@ sub findRecords($%)
 	return undef;
     }
 
-    my $int=new EasyDBI::RecordIterator($sth);
+    my $int=new IMDB::Local::DB::RecordIterator($sth);
     $int->{cacheBy}=$cacheBy;
     return($int);
 }
 
 =head2 rowExists
 
+Check to see at least one row exists with value in 'column' in the specified 'table'
+
 =cut
 
 sub rowExists($$$)
 {
     my ($self, $table, $column, $value)=@_;
-    my $sql="SELECT $column from $table where $column='$value'";
+    my $sql;
+    if ( $self->driver eq 'ODBC' ) {
+	$sql="SELECT TOP 1 $column from $table where $column='$value'";
+    }
+    elsif ( $self->driver eq 'mysql' ) {
+	$sql="SELECT $column from $table where $column='$value' LIMIT 1";
+    }
+    elsif ( $self->driver eq 'SQLite' ) {
+	$sql="SELECT $column from $table where $column='$value' LIMIT 1";
+    }
 
     my $v=$self->select2Scalar($sql);
     return(defined($v) && $v eq $value);
@@ -628,6 +687,8 @@ sub rowExists($$$)
 
 =head2 select2Scalar
 
+Execute the given sql statement and return the value in a single scalar value
+
 =cut
 
 sub select2Scalar($$)
@@ -644,6 +705,8 @@ sub select2Scalar($$)
 
 =head2 select2Int
 
+Execute the given sql statement and return the value cast as an integer (ie int(returnvalue))
+
 =cut
 
 sub select2Int($$)
@@ -659,6 +722,8 @@ sub select2Int($$)
 
 =head2 select2Array
 
+Execute the given sql statement and return an array with all the results.
+
 =cut
 
 sub select2Array($$)
@@ -679,6 +744,8 @@ sub select2Array($$)
 
 =head2 select2Matrix
 
+Execute the given sql statement and return an array of arrays, each containing a row of values
+
 =cut
 
 sub select2Matrix($$)
@@ -689,7 +756,7 @@ sub select2Matrix($$)
     if ( !$sth ) {
 	return(undef);
     }
-    my $int=new EasyDBI::RecordIterator($sth);
+    my $int=new IMDB::Local::DB::RecordIterator($sth);
     my @arr;
     while (my $refer=$int->nextRow()) {
 	push(@arr, $refer);
@@ -699,6 +766,8 @@ sub select2Matrix($$)
 
 =head2 select2HashRef
 
+Execute the given sql statement and return a reference to a hash with the result
+
 =cut
 
 sub select2HashRef($$)
@@ -714,6 +783,8 @@ sub select2HashRef($$)
 
 =head2 select2Hash
 
+Execute the given sql statement and return a reference ot a hash containing the given row.
+
 =cut
 
 sub select2Hash($$)
@@ -724,7 +795,7 @@ sub select2Hash($$)
     if ( !$sth ) {
 	return(undef);
     }
-    my $int=new EasyDBI::RecordIterator($sth);
+    my $int=new IMDB::Local::DB::RecordIterator($sth);
     my %arr;
     while (my $refer=$int->nextRow()) {
 	my $key=$refer->[0];
@@ -739,15 +810,10 @@ sub select2Hash($$)
     return(\%arr);
 }
 
-=head2 database_list
-
-=cut
-
-sub database_list($)
+sub _NOT_USED_database_list($)
 {
     my $self=shift;
 
-    my $sth;
     if ( $self->driver eq 'ODBC' ) {
 	my @list;
 	for my $t (@{$self->select2Array("select Name from master..sysdatabases")} ) {
@@ -771,23 +837,32 @@ sub database_list($)
 
 =head2 table_list
 
+Retrieve a list of tables available. Created Tables or Views created after connect() may not be included.
+
 =cut
 
 sub table_list($)
 {
     my $self=shift;
 
+    # already got an answer ?
     if ( $self->tables_index(0) ) {
 	return($self->tables);
     }
 
-    my $sth;
     if ( $self->driver eq 'ODBC' ) {
-	$sth=$self->dbh()->table_info($self->database, '', '', 'TABLE');
+        my $sth=$self->dbh()->table_info($self->database, '', '', 'TABLE');
 	if ( !defined($sth) ) {
 	    warn("lookup of table_info failed:".$self->dbh->errstr()."\n");
 	    return($self->tables);
 	}
+	my $int=new IMDB::Local::DB::RecordIterator($sth);
+	while (my $refer=$int->nextRow()) {
+	    my ($table_cat, $table_schem, $table_name, $table_type, $remarks)=@$refer;
+	    $self->tables_push($table_name);
+	}
+	#warn "tables:".join(',', $self->tables)."\n";
+	return($self->tables);
     }
     elsif ( $self->driver eq 'SQLite' ) {
 	#$sth=$self->dbh()->table_info('%', '%', '%', 'TABLE');
@@ -803,18 +878,13 @@ sub table_list($)
 	}
 	return($self->tables);
     }
-    
-    my $int=new EasyDBI::RecordIterator($sth);
-    while (my $refer=$int->nextRow()) {
-	my ($table_cat, $table_schem, $table_name, $table_type, $remarks)=@$refer;
-	$self->tables_push($table_name);
-    }
-    #warn "tables:".join(',', $self->tables)."\n";
-    return($self->tables);
+   
 }
 
 =head2 table_exists
 
+Check to see if a given table exists. Uses table_list.
+
 =cut
 
 sub table_exists($$)
@@ -829,6 +899,15 @@ sub table_exists($$)
 
 =head2 column_info
 
+Retrieve information about a given column in a table. Changes to columns made after connect() may not be included.
+
+Returns a list of columns in a database/driver specific order containing:
+ COLUMN_NAME - name of the column
+ TYPE_NAME - data type (if available)
+ COLUMN_SIZE - size of column data (if available)
+ IS_NULL - true/false if column is nullable
+ IS_PRIMARY_KEY - 1 or 0 if column is a primary key
+
 =cut
 
 sub column_info($$)
@@ -919,6 +998,8 @@ sub column_info($$)
 
 =head2 column_list
 
+Retrieve a list of column names in column_list order.
+
 =cut
 
 sub column_list($$)
@@ -934,6 +1015,8 @@ sub column_list($$)
 
 =head2 writeQuery2CSV
 
+Run an sql query and output the result to the specified file in Text::CSV format
+
 =cut
 
 sub writeQuery2CSV($$$)
@@ -962,6 +1045,12 @@ sub writeQuery2CSV($$$)
 
 =head2 appendCSV2Table
 
+Parse the given CVS file (which must have column names that match a the given table) and insert each row
+as a new row into the specified table.
+
+Upon success, returns > 0, number of rows successfully inserted.
+Returns 0 if open() on the given file fails.
+
 =cut
 
 sub appendCSV2Table($$$)
@@ -974,6 +1063,7 @@ sub appendCSV2Table($$$)
     if ( open(my $fd, "<:encoding(utf8)", $file) ) {
 	my $lineNum=1;
         my @titleRow=@{$csv->getline($fd)};
+	my $rowsInserted=0;
 	
         while ( my $row = $csv->getline($fd) ) {
 	    $lineNum++;
@@ -987,20 +1077,24 @@ sub appendCSV2Table($$$)
 		    $args{$t}=$row->[$c];
 		    $c++;
 		}
-		$self->insert_db($table, undef, %args);
+		if ( $self->insert_row($table, undef, %args) ) {
+		    $rowsInserted++;
+		}
 	    }
         }
 	
         $csv->eof or $csv->error_diag();
-        close $fd;
+        close($fd);
 	$self->commit();
-	return(1);
+	return($rowsInserted);
     }
     return(0);
 }
 
 =head2 table_row_count
 
+Retrieve the # of rows in a given table.
+
 =cut
 
 sub table_row_count($$)
@@ -1022,7 +1116,7 @@ sub table_row_count($$)
 	    return(0);
 	}
 	
-	my $int=new EasyDBI::RecordIterator($sth);
+	my $int=new IMDB::Local::DB::RecordIterator($sth);
 	my @infos;
 	while (my $r=$int->nextRow()) {
 	    my ($table, $rows, $reserved_kb, $data_kb, $index_kb, $unused_kb)=@$r;
@@ -1034,6 +1128,8 @@ sub table_row_count($$)
 
 =head2 table_report
 
+Retrieve a reference to an array of arrays, each sub-array containing [table, #ofRows, data-size-in-KBs, index-size-in-KBs]
+
 =cut
 
 sub table_report($@)
@@ -1070,7 +1166,7 @@ sub table_report($@)
     my $sql="sp_Msforeachtable 'sp_spaceused ''?'''";
     my $sth=$self->execute($sql);
     
-    my $int=new EasyDBI::RecordIterator($sth);
+    my $int=new IMDB::Local::DB::RecordIterator($sth);
     my @infos;
     while (my $r=$int->nextRow()) {
 	my ($table, $rows, $reserved_kb, $data_kb, $index_kb, $unused_kb)=@$r;
@@ -1091,7 +1187,7 @@ sub table_report($@)
     
 =head1 AUTHOR
 
-jerryv, C<< <jerryv at cpan.org> >>
+jerryv, C<< <jerry.veldhuis at gmail.com> >>
 
 =head1 BUGS
 
@@ -1099,9 +1195,6 @@ Please report any bugs or feature requests to C<bug-imdb-local at rt.cpan.org>,
 the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=IMDB-Local>.  I will be notified, and then you'll
 automatically be notified of progress on your bug as I make changes.
 
-
-
-
 =head1 SUPPORT
 
 You can find documentation for this module with the perldoc command.
